/**
 * CimandeActionInvocation.java
 *
 * Created on 5 Sep 2010 04:44:06
 */
package org.blueoxygen.cimande.rest;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts2.ServletActionContext;
import org.blueoxygen.cimande.rest.handler.ContentHandler;
import org.blueoxygen.cimande.rest.manager.ContentHandlerManager;

import com.opensymphony.xwork2.DefaultActionInvocation;
import com.opensymphony.xwork2.ModelDriven;
import com.opensymphony.xwork2.Result;
import com.opensymphony.xwork2.config.entities.ActionConfig;
import com.opensymphony.xwork2.inject.Inject;
import com.opensymphony.xwork2.util.ValueStack;
import com.opensymphony.xwork2.util.logging.Logger;
import com.opensymphony.xwork2.util.logging.LoggerFactory;
import com.opensymphony.xwork2.util.profiling.UtilTimerStack;

/**
 * @author Dian Aditya
 * 
 */
public class CimandeRestActionInvocation extends DefaultActionInvocation {

	private static final Logger LOG = LoggerFactory
			.getLogger(CimandeRestActionInvocation.class);

	private ContentHandlerManager manager;

	/**
	 * @param extraContext
	 * @param pushAction
	 */
	public CimandeRestActionInvocation(Map<String, Object> extraContext,
			boolean pushAction) {
		super(extraContext, pushAction);
	}

	@Inject
	public void setHandler(ContentHandlerManager manager) {
		this.manager = manager;
	}

	protected String invokeAction(Object object, ActionConfig config)
			throws Exception {

		ValueStack valueStack = invocationContext.getValueStack();

		for (String param : invocationContext.getParameters().keySet()) {
			Object objectParameter = valueStack.findValue(param);
			if (objectParameter != null) {
				if (objectParameter.getClass().isArray()) {
					if (((Object[]) objectParameter).length == 1) {
						valueStack.setValue(param,
								((Object[]) objectParameter)[0]);
					}
				}
			}
		}

		String methodName = proxy.getMethod();
		HttpServletRequest request = ServletActionContext.getRequest();

		if (LOG.isDebugEnabled()) {
			LOG.debug("Executing action method = " + config.getMethodName());
		}

		String timerKey = "invokeAction: " + proxy.getActionName();
		try {
			UtilTimerStack.push(timerKey);

			boolean methodCalled = false;
			Object methodResult = null;
			Method method = null;
			try {
				String httpMethod = request.getMethod().toLowerCase();
				httpMethod = httpMethod.substring(0, 1).toUpperCase()
						+ httpMethod.substring(1);

				// Find method + httpMethod
				try {
					method = getAction().getClass().getMethod(methodName,
							new Class[0]);
				} catch (NoSuchMethodException e) {
					if (methodName.contains(httpMethod)) {
						methodName = methodName.substring(0,
								methodName.lastIndexOf(httpMethod));
						method = getAction().getClass().getMethod(methodName,
								new Class[0]);
					}
				}
			} catch (NoSuchMethodException e) {
				// hmm -- OK, try doXxx instead
				try {
					String altMethodName = "do"
							+ methodName.substring(0, 1).toUpperCase()
							+ methodName.substring(1);
					method = getAction().getClass().getMethod(altMethodName,
							new Class[0]);
				} catch (NoSuchMethodException e1) {
					// well, give the unknown handler a shot
					if (unknownHandlerManager.hasUnknownHandlers()) {
						try {
							methodResult = unknownHandlerManager
									.handleUnknownMethod(action, methodName);
							methodCalled = true;
						} catch (NoSuchMethodException e2) {
							// throw the original one
							throw e;
						}
					} else {
						throw e;
					}
				}
			}

			if (!methodCalled) {
				methodResult = method.invoke(action, new Object[0]);
			}

			if (methodResult instanceof Result) {
				this.explicitResult = (Result) methodResult;

				// Wire the result automatically
				container.inject(explicitResult);
				return null;
			} else if (methodResult != null) {

				// Begin write content metadata
				String path = request.getServletPath().substring(1);
				String[] url = path.split("/");

				Object model = null;
				if (object instanceof ModelDriven) {
					model = ((ModelDriven) object).getModel();
				} else {
					model = object;
				}

				if (model instanceof Map) {
					Map map = (Map) model;
					if (map.get("root") != null)
						model = map.get("root");
				}

				if (url.length > 1
						&& manager.getHandlerByExtension().containsKey(
								url[url.length - 1])) {
					ContentHandler handler = manager.getHandlerByExtension()
							.get(url[url.length - 1]);
					manager.writeObject(handler, model);
				}

				return (String) methodResult;
				// End
			}

			return resultCode;
		} catch (NoSuchMethodException e) {
			throw new IllegalArgumentException("The " + methodName
					+ "() is not defined in action " + getAction().getClass()
					+ "");
		} catch (InvocationTargetException e) {
			// We try to return the source exception.
			Throwable t = e.getTargetException();

			if (actionEventListener != null) {
				String result = actionEventListener.handleException(t,
						getStack());
				if (result != null) {
					return result;
				}
			}
			if (t instanceof Exception) {
				throw (Exception) t;
			} else {
				throw e;
			}
		} finally {
			UtilTimerStack.pop(timerKey);
		}
	}
}
