package org.blueoxygen.cimande.role.actions.servicewfrole;

import java.util.List;

import org.blueoxygen.cimande.persistence.hibernate.dao.HibernatePersistenceDaoManager;
import org.blueoxygen.cimande.role.Role;
import org.hibernate.Criteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/*******************************
 * @author Praditya Eldorado 
 *         aldo.donal@gmail.com
 *******************************/

@Repository
@Transactional(readOnly = true)
public class ServiceWfRoleImpl extends HibernatePersistenceDaoManager<Role>
		implements ServiceWfRole {
	@Transactional
	public void save(Role role) {
		if (role != null) {
			if (role.getId() == null || "".equalsIgnoreCase(role.getId())) {
				role.getLogInformation().setCreateBy(getCurrentUser().getId());
				role.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				role.getLogInformation().setCreateDate(getCurretTime());
				role.getLogInformation().setLastUpdateDate(getCurretTime());
				role.getLogInformation().setActiveFlag(1);
				persist(role);
			} else {
				role.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				role.getLogInformation().setLastUpdateDate(getCurretTime());

				merge(role);
			}

		}

	}

	@Transactional
	public void delete(Role role) {
		if (role != null && role.getId() != null) {
			role = getById(Role.class, role.getId());

			remove(role);
		}
	}

	@Transactional(readOnly = true)
	public Role getRoleById(String id) {
		return getById(Role.class, id);
	}

	@Transactional(readOnly = true)
	public List<Role> getAllRoles(String orderby) {
		return findAllSorted(Role.class, orderby);
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<Role> findRoles(String name, String description,
			int activeFlag, String orderby, int maxResults, int firstResult) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Role.class);

		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("description", description + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.eq("logInformation.activeFlag", activeFlag));
		criteria.setMaxResults(maxResults);
		criteria.setFirstResult(firstResult);
		criteria.addOrder(Order.asc(orderby));
		return criteria.list();
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<Role> findRolesByField(String field, String value) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Role.class);
		criteria.add(Restrictions.like(field, value));

		return criteria.list();
	}

	@Transactional(readOnly = true)
	public int getRowCountByCriteria(String name, String description) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Role.class);

		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("description", description + "",
				MatchMode.ANYWHERE));
		criteria.setProjection(Projections.rowCount());

		return new Integer(criteria.uniqueResult() + "");
	}

	public List<Role> findRoless(String name, String description) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Role.class);
		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("description", description + "",
				MatchMode.ANYWHERE));
		return criteria.list();
	}

}
