package org.blueoxygen.cimande.roleprivilage.service;

import java.util.ArrayList;
import java.util.List;

import org.blueoxygen.cimande.modulefunction.ModuleFunction;
import org.blueoxygen.cimande.persistence.hibernate.dao.HibernatePersistenceDaoManager;
import org.blueoxygen.cimande.role.Role;
import org.blueoxygen.cimande.role.RolePrivilage;
import org.hibernate.Criteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/*******************************
 * @author Praditya Eldorado aldo.donal@gmail.com
 *******************************/

@Repository
@Transactional(readOnly = true)
public class RolePrivilageServiceImpl extends
		HibernatePersistenceDaoManager<RolePrivilage> implements
		RolePrivilageService {

	@Transactional
	public void save(RolePrivilage rolePrivilage) {
		if (rolePrivilage != null) {
			if (rolePrivilage.getId() == null
					|| "".equalsIgnoreCase(rolePrivilage.getId())) {
				rolePrivilage.getLogInformation().setCreateBy(
						getCurrentUser().getId());
				rolePrivilage.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				rolePrivilage.getLogInformation()
						.setCreateDate(getCurretTime());
				rolePrivilage.getLogInformation().setLastUpdateDate(
						getCurretTime());
				rolePrivilage.getLogInformation().setActiveFlag(1);

				persist(rolePrivilage);
			}
		}
	}

	@Transactional
	public void delete(RolePrivilage rolePrivilage) {
		if (rolePrivilage != null && rolePrivilage.getId() != null) {
			rolePrivilage = getById(RolePrivilage.class, rolePrivilage.getId());

			remove(rolePrivilage);
		}
	}

	@Transactional(readOnly = true)
	public RolePrivilage getRolePrivilageById(String id) {
		return getById(RolePrivilage.class, id);
	}

	@Transactional(readOnly = true)
	public List<RolePrivilage> getAllRolePrivilages() {
		return findAll(RolePrivilage.class);
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<RolePrivilage> findRolePrivilages(String Url_xml,
			String Privilage_flag, String Role, String ModuleFunction,
			int maxResults, int firstResult) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RolePrivilage.class);
		criteria.add(Restrictions.like("Url_xml", Url_xml + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("Privilage_flag", Privilage_flag + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("Role", Role + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("ModuleFunction", ModuleFunction + "",
				MatchMode.ANYWHERE));

		criteria.setMaxResults(maxResults);
		criteria.setFirstResult(firstResult);

		return criteria.list();

	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<RolePrivilage> findRolePrivilagesByField(String field,
			String value) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RolePrivilage.class);

		criteria.add(Restrictions.like(field, value));

		return criteria.list();
	}

	public int getRowCountByCriteria(String Url_xml, String Privilage_flag,
			String Role, String ModuleFunction) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RolePrivilage.class);

		criteria.add(Restrictions.like("Url_xml", Url_xml + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("Privilage_flag", Privilage_flag + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("Role", Role + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("ModuleFunction", ModuleFunction + "",
				MatchMode.ANYWHERE));

		criteria.setProjection(Projections.rowCount());

		return new Integer(criteria.uniqueResult() + "");
	}

	public int getRowCountByCriteria(Role role, ModuleFunction moduleFunction,
			String Role, int i) {

		return 0;
	}

	public List findRolePrivilages(Role role, ModuleFunction moduleFunction,
			String url_xml, int privilage_flag, String orderBy,
			int maxRowPerPage, int i) {
		// TODO Auto-generated method stub
		return null;
	}

	@Transactional(readOnly = true)
	public List<RolePrivilage> getRolePrivilages(Role role) {
		if (role == null)
			return new ArrayList<RolePrivilage>();

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RolePrivilage.class);
		criteria.add(Restrictions.eq("role.id", role.getId()));

		return criteria.list();
	}

	@Transactional(readOnly = true)
	public List<RolePrivilage> getList(String roleId, int activeFlag) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RolePrivilage.class);
		criteria.add(Restrictions.like("role.id", roleId));
		criteria.add(Restrictions.like("logInformation.activeFlag", activeFlag));
		return criteria.list();
	}

	public List<RolePrivilage> getLists(String parentId, String roleId,
			int activeFlag) {
		activeFlag = 1;
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RolePrivilage.class);
		criteria.add(Restrictions.or(Restrictions.like("role.id", roleId),
				Restrictions.like("role.id", parentId)));
		criteria.add(Restrictions.like("logInformation.activeFlag", activeFlag));
		return criteria.list();
	}
}
