/**
 * 
 */
package org.blueoxygen.cimande.sitemanager.yuitree.treenode;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts2.ServletActionContext;
import org.blueoxygen.cimande.modulefunction.ModuleFunction;
import org.blueoxygen.cimande.privilege.Privilege;
import org.blueoxygen.cimande.role.RolePrivilage;
import org.blueoxygen.cimande.role.RoleSite;
import org.blueoxygen.cimande.role.RoleSitePrivilage;
import org.blueoxygen.cimande.security.LoginFilter;
import org.blueoxygen.cimande.security.User;
import org.blueoxygen.cimande.sitemanager.yuitree.TreeNode;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author Dian Aditya
 * 
 */
@Repository
@Transactional(readOnly = true)
public class TreeNodeFactoryImpl implements TreeNodeFactory {

	@Autowired
	private SessionFactory factory;

	@SuppressWarnings("unchecked")
	public List<TreeNode> getTreeNodes(User user) {
		HttpServletRequest request = ServletActionContext.getRequest();

		Session session = factory.getCurrentSession();
		List<TreeNode> treeNodes = new ArrayList<TreeNode>();

		String roleId = user.getRole().getId();
		String siteId = (String) ServletActionContext.getRequest().getSession()
				.getAttribute(LoginFilter.LOGIN_CIMANDE_SITE);

		Criteria criteria = session.createCriteria(RoleSite.class);
		criteria.add(Restrictions.eq("role.id", roleId));
		criteria.add(Restrictions.eq("site.id", siteId));

		RoleSite roleSite = (RoleSite) criteria.uniqueResult();
		List<Privilege> privileges = new ArrayList<Privilege>();

		Map<String, ModuleFunction> moduleFunctionMap = new HashMap<String, ModuleFunction>();

		if (roleSite != null) {
			Query query = session
					.createQuery("FROM "
							+ RoleSitePrivilage.class.getName()
							+ " s WHERE s.roleSite.id = :id ORDER BY s.moduleFunction.description");
			query.setParameter("id", roleSite.getId());

			privileges = query.list();
		} else {
			Query query = session
					.createQuery("FROM "
							+ RolePrivilage.class.getName()
							+ " s WHERE s.role.id = :id ORDER BY s.moduleFunction.description");
			query.setParameter("id", roleSite.getId());

			privileges = query.list();
		}

		for (Privilege privilege : privileges) {
			ModuleFunction moduleFunction = privilege.getModuleFunction();

			TreeNode node = null;

			if (moduleFunction.getModuleDescriptor() != null
					&& moduleFunction.getModuleDescriptor().getTypeFlag() == 1) {
				node = new TreeNode(moduleFunction.getId(),
						moduleFunction.getDescription(), true);
			} else {
				node = new TreeNode(moduleFunction.getId(),
						moduleFunction.getDescription(), false);
			}

			treeNodes.add(node);
			moduleFunctionMap.put(moduleFunction.getId(), moduleFunction);
		}

		request.getSession().setAttribute("mfs", moduleFunctionMap);

		return treeNodes;
	}

	@SuppressWarnings("unchecked")
	public List<TreeNode> getTreeChild(String nodeId) {
		HttpServletRequest request = ServletActionContext.getRequest();

		Criteria criteria = factory.getCurrentSession().createCriteria(
				ModuleFunction.class);
		criteria.add(Restrictions.eq("moduleFunction.id", nodeId));
		criteria.addOrder(Order.asc("description"));

		List<ModuleFunction> moduleFunctions = criteria.list();
		List<TreeNode> treeNodes = new ArrayList<TreeNode>();

		if (moduleFunctions.size() > 0) {
			List<TreeNode> leaf = new ArrayList<TreeNode>();

			for (ModuleFunction function : moduleFunctions) {
				TreeNode node = null;

				if (function.getModuleDescriptor() != null
						&& function.getModuleDescriptor().getTypeFlag() == 1) {
					node = new TreeNode(function.getId(),
							function.getDescription(), true);

					node.setHref("#" + "/module/"
							+ function.getModuleDescriptor().getName());

					leaf.add(node);
				} else {
					node = new TreeNode(function.getId(),
							function.getDescription(), false);

					treeNodes.add(node);
				}

				((Map<String, ModuleFunction>) request.getSession()
						.getAttribute("mfs")).put(function.getId(), function);
			}

			treeNodes.addAll(leaf);
		} else {
			ModuleFunction function = (ModuleFunction) factory
					.getCurrentSession().get(ModuleFunction.class, nodeId);

			String sActionFlag = "", sUrlAction = "";
			/*
			 * check the record in descriptor table if the action_flag = 1 means
			 * url_action else the node is url_descriptor value
			 */
			sActionFlag = String.valueOf(function.getModuleDescriptor()
					.getTypeFlag());

			if (sActionFlag.equals("1")) {
				// sUrlAction = function.getModuleDescriptor().getUrlAction()
				// + "&";
				sUrlAction = "#" + "/module/"
						+ function.getModuleDescriptor().getName() + "/";
			} else if (sActionFlag.equals("2")) {
				sUrlAction = /* request.getContextPath() */"#" + "/module/"
						+ function.getModuleDescriptor().getName() + "/";
			} else if (sActionFlag.equals("3")) {
				sUrlAction = /* request.getContextPath() */"#"
						+ "/module/window/genform?window.id="
						+ function.getModuleDescriptor().getWindow().getId()
						+ "&";
			} else {
				sUrlAction = /* request.getContextPath() */"#" + "/descriptor/"
						+ function.getModuleDescriptor().getUrlDescriptor()
						+ "?";
			}

			if (!sActionFlag.equals("1")) {
				TreeNode node = new TreeNode(function.getId() + "create",
						"New", true);

				if (sActionFlag.equals("2")) {
					node.setHref(sUrlAction + "create");
					// node.setHrefTarget("main");
				} else {
					node.setHref(sUrlAction + "action=new");
					// node.setHrefTarget("main");
				}

				treeNodes.add(node);

				node = new TreeNode(function.getId() + "filter", "Search", true);

				if (sActionFlag.equals("2")) {
					node.setHref(sUrlAction + "filter");
					// node.setHrefTarget("main");
				} else {
					node.setHref(sUrlAction + "action=search");
					// node.setHrefTarget("main");
				}

				treeNodes.add(node);
			}

			((Map<String, ModuleFunction>) request.getSession().getAttribute(
					"mfs")).put(function.getId(), function);
		}

		return treeNodes;
	}

	@SuppressWarnings("unchecked")
	public boolean hasAccess(String descriptor) {
		HttpServletRequest request = ServletActionContext.getRequest();
		Session session = factory.getCurrentSession();

		Map<String, ModuleFunction> moduleFunctions = (Map<String, ModuleFunction>) request
				.getSession().getAttribute("mfs");

		for (ModuleFunction function : moduleFunctions.values()) {
			if (function.getModuleDescriptor() != null) {
				if (function.getModuleDescriptor().getName().equals(descriptor))
					return true;
			}
		}

		String sql = "FROM " + ModuleFunction.class.getName()
				+ " m WHERE m.moduleDescriptor.name = :descriptor";
		Query query = session.createQuery(sql).setParameter("descriptor",
				descriptor);

		List<ModuleFunction> functions = query.list();

		if (functions.size() < 1)
			return false;

		for (ModuleFunction function : functions) {
			ModuleFunction temp = function;

			while (temp.getModuleFunction() != null) {
				if (temp.getModuleFunction().getId().equals("0")) {
					break;
				} else {
					temp = temp.getModuleFunction();
				}
			}

			if (moduleFunctions.containsKey(temp.getId())) {
				((Map<String, ModuleFunction>) request.getSession()
						.getAttribute("mfs")).put(temp.getId(), temp);

				return true;
			}
		}

		return false;
	}
}
