/**
 * Copyright 2011 BlueOxygen Technology
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.blueoxygen.cimande.social.controller;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import javax.inject.Inject;

import org.blueoxygen.cimande.commons.CimandeModuleAction;
import org.blueoxygen.cimande.role.UserRole;
import org.blueoxygen.cimande.security.Address;
import org.blueoxygen.cimande.security.LoginFilter;
import org.blueoxygen.cimande.security.Name;
import org.blueoxygen.cimande.security.User;
import org.blueoxygen.cimande.security.login.interceptors.LoginInterceptor;
import org.blueoxygen.cimande.security.usermanager.service.UserSiteService;
import org.blueoxygen.cimande.social.SocialAccount;
import org.blueoxygen.cimande.social.service.SocialAccountService;
import org.blueoxygen.util.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.social.connect.Connection;
import org.springframework.social.facebook.api.Facebook;
import org.springframework.social.facebook.api.FacebookProfile;
import org.springframework.social.facebook.connect.FacebookConnectionFactory;
import org.springframework.social.oauth2.AccessGrant;
import org.springframework.social.oauth2.GrantType;
import org.springframework.social.oauth2.OAuth2Operations;
import org.springframework.social.oauth2.OAuth2Parameters;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

/**
 * @author Dian Aditya
 * 
 */
public class FacebookSignInController extends CimandeModuleAction {

	@Inject
	private FacebookConnectionFactory facebookConnectionFactory;

	@Inject
	private SocialAccountService accountService;

	@Inject
	private UserSiteService userSiteService;

	@Value("${social.facebook.redirectUri}")
	private String redirectUri;
	@Value("${social.facebook.scope}")
	private String scope;
	@Value("${social.facebook.state}")
	private String state;

	public String signInPost() throws Exception {
		OAuth2Operations operations = facebookConnectionFactory
				.getOAuthOperations();

		// Map<String, String> additionalParameters = new HashMap<String,
		// String>();
		// additionalParameters.put("display",
		// Arrays.asList(new String[] { "page" }));

		// OAuth2Parameters parameters = new OAuth2Parameters(redirectUri,
		// scope,
		// state, additionalParameters);

		OAuth2Parameters parameters = new OAuth2Parameters();
		parameters.setRedirectUri(redirectUri);
		parameters.setScope(scope);
		parameters.setState(state);
		parameters.set("display", "page");

		model.put("authorizeUrl", operations.buildAuthorizeUrl(
				GrantType.AUTHORIZATION_CODE, parameters));

		return "redirect";
	}

	public String signInGet() throws IOException {
		AccessGrant grant = facebookConnectionFactory.getOAuthOperations()
				.exchangeForAccess(model.get("code") + "",
						request.getRequestURL().toString(), null);
		Connection<Facebook> connection = facebookConnectionFactory
				.createConnection(grant);

		Facebook facebook = connection.getApi();

		if (facebook.isAuthorized()) {
			User user = accountService.getUserBySocialAccount("facebook",
					facebook.userOperations().getUserProfile().getUsername());

			FacebookProfile profile = facebook.userOperations()
					.getUserProfile();

			if (user == null) {
				user = accountService.getUserByEmail(profile.getEmail());

				if (user == null) {
					user = new User();
					user.setId(null);
					user.setName(new Name());
					user.setAddress(new Address());

					user.setUsername(profile.getUsername());
					user.setPassword("");
					user.setEmail(profile.getEmail());
					user.getName().setFirst(profile.getFirstName());
					user.getName().setLast(profile.getLastName());
					user.getAddress().setCity(profile.getHometown().getName());
					user.setRole(accountService.getDefaultRole());
					user.getAddress().setCity(profile.getLocation().getName());
					user.getAddress().setStreet1(
							profile.getLocale().getCountry());
					user.setWorkspace_type("tree");
				}

				SocialAccount socialAccount = new SocialAccount();
				socialAccount.setSocialProvider("facebook");
				socialAccount.setUser(user);
				socialAccount.setUsername(profile.getUsername());
				socialAccount.setSocialId(profile.getId());

				accountService.addNewUserBySocialAccount(socialAccount);
			}

			if (user.getRole().getName().equalsIgnoreCase("default")) {
				return LOGIN;
			} else {
				request.getSession().setAttribute(
						LoginFilter.LOGIN_CIMANDE_USER,
						new StringUtils().encodeBase64(user.getId()));
				request.getSession().setAttribute(
						LoginInterceptor.CIMANDE_USER, user);

				return SUCCESS;
			}
		}

		return SUCCESS;
	}
}
